from analogs_finder.search_methods import methods as mt
from analogs_finder.search_methods import molecule as ml
from analogs_finder.search_methods import similarity as sm



def turbo_similarity(mref, molecules, neighbours=5, treshold=0.7, fp_type="DL"):
    """
    Turbo similarity searching uses information 
    about the nearest neighbors in a conventional 
    chemical similarity search to increase the 
    effectiveness of virtual screening with a data 
    fusion approach being used to combine the 
    nearest neighbor information. 
    Paper: https://onlinelibrary.wiley.com/doi/abs/10.1002/sam.10037
    """
    mol_most_similars  = mt.search_most_similars(mref, molecules, neighbours, fp_type=fp_type)
    mol_most_similars.insert(0, ml.Molecule(mref))
    molecules_ref = [ m.molecule for m in mol_most_similars ]
    for m in sm.compute_similarity_several_mols(molecules_ref, molecules, fp_type=fp_type):
        if any(s > float(treshold) for s in m.similarities):
            sim_min = min(m.similarities)
            m.molecule.SetProp("Similarity", "{}".format(sim_min))
            yield m
        else:
            pass



import argparse
import sys
from functools import partial
import collections
from tqdm import tqdm
import numpy as np
from rdkit import Chem
from rdkit import DataStructs
from multiprocessing import Pool
from analogs_finder.search_methods import fingerprints as fps
from analogs_finder.search_methods import similarity as sm
from analogs_finder.search_methods import molecule as ml

def search_most_similars(molecule_query, molecules_db, n_structs, fp_type="DL"):
    molecules_most_similar = [0] * n_structs
    similarity = np.zeros(n_structs)
    for m in tqdm(sm.compute_similarity(molecule_query, molecules_db, fp_type)):
        idx = np.argmin(similarity)
        less_similar = similarity[idx]
        if m.similarity > less_similar:
            similarity[idx] = m.similarity
            m.molecule.SetProp("Similarity", str(m.similarity))
            molecules_most_similar[idx] = m
    molecules_most_similar = [ m for m in molecules_most_similar if m != 0 ]
    return molecules_most_similar

def search_similarity_tresh(molecule_query, molecules_db, treshold, fp_type="DL"):
    for m in tqdm(sm.compute_similarity(molecule_query, molecules_db, fp_type)):
        if m.similarity > float(treshold):
            m.molecule.SetProp("Similarity", str(m.similarity))
            yield m

def search_similarity_tresh_several_fp(molecule_query, molecules_db, tresholds=[0.7, 0.4, 0.7, 0.4], fp_types=["DL", "circular", "torsions", "MACCS"]):
    assert len(tresholds) == len(fp_types), "Number of fingerprints must be equal to number of tresholds"
    print("Searching most similars several fingerprint types...")
    for m in sm.compute_similarity_severl_fp(molecule_query, molecules_db, fp_types, tresholds):
        m.molecule.SetProp("Similarity_{}".format(m.fp_type), str(m.similarity))
        yield m

def search_substructure(molecule_query, molecules_db):
    print("Searching for substructure")
    all_substructs_found = True
    for i, m in tqdm(enumerate(molecules_db)):
        if not m:
            print("Skipping {}".format(i))
            continue
        all_substructs_found = True
        for m_ref in molecule_query:
            if not m.HasSubstructMatch(m_ref, useChirality=True):
                all_substructs_found = False
        if all_substructs_found:
            yield ml.molec_properties(m)

def combi_substructure_search(sdfs, molecules_db):
    print("Searching for substructure")
    for i, m in tqdm(enumerate(molecules_db)):
        if not m:
            print("Skipping {}".format(i))
            continue
        substructs_found = [False] * len(sdfs)
        for i, sdf in enumerate(sdfs):
            for m_ref in Chem.SDMolSupplier(sdf):
                if m.HasSubstructMatch(m_ref, useChirality=True):
                    substructs_found[i] = True
        if all(substructs_found):
            yield ml.molec_properties(m)
 

def most_similar_with_substructure(molecule_query, molecules_db, substructures, treshold, fp_type="DL"):
    for m in tqdm(sm.compute_similarity(molecule_query, molecules_db, fp_type)):
        # Similarity based
        if m.similarity > float(treshold):
            for substruct in Chem.SDMolSupplier(substructures):
                # Substructure based
                if m.molecule.HasSubstructMatch(substruct, useChirality=True):
                    m.molecule.SetProp("Similarity", str(m.similarity))
                    yield m

    



from rdkit.Chem.Fingerprints import FingerprintMols
from rdkit.Chem import MACCSkeys
from rdkit.Chem.AtomPairs import Pairs
from rdkit.Chem import AllChem
from rdkit.Chem.Pharm2D import Gobbi_Pharm2D,Generate


def fingerprint(mol, fp_type="DL"):
    if fp_type == "DL":
        return FingerprintMols.FingerprintMol(mol)
    elif fp_type == "circular":
        return AllChem.GetMorganFingerprintAsBitVect(mol, 3, nBits=1024)
    elif fp_type == "MACCS":
        return MACCSkeys.GenMACCSKeys(mol)
    elif fp_type == "torsions":
        return Pairs.GetAtomPairFingerprintAsBitVect(mol)
    elif fp_type == "pharm":
        return Generate.Gen2DFingerprint(mol,Gobbi_Pharm2D.factory) 




import os

def sdf_to_mae(sdf, output=None):
    if not output:
        output = os.path.splitext(os.path.basename(sdf))[0]+".mae"
    sdconvert = os.path.join(os.environ["SCHRODINGER"], "utilities/sdconvert")
    command = "{} -isdf {}  -omae {} > /dev/null".format(sdconvert, sdf, output)
    print(command)
    os.system(command)
    return output



import os
import argparse
from analogs_finder.helpers import formats as ft



class PhaseScreen():

    def __init__(self, query_mol, database):
        self.query_mol = query_mol
        self.database = database

    
    def create_pharmacophore_hypotesis(self, output="query_molec.phypo"):
        hypotesis_bin =  os.path.join(os.environ["SCHRODINGER"], "utilities/create_hypoFiles")
        project_name = output.split(".")[:-1]
        command = "{} {} {} > /dev/null".format(hypotesis_bin, self.query_mol, project_name[0])
        print(command)
        os.system(command)
        self.hypotesis = output
        return self.hypotesis 
   
    def run(self, output_project="query_molec_phase", minimization=True, match=None):
        assert hasattr(self, "hypotesis"), "create_pharmacophore_hypotesis must be run before phase screening"
        phase_bin = os.path.join(os.environ["SCHRODINGER"], "phase_screen")
        command = "{} {} {} {} -flex".format(phase_bin, self.database, self.hypotesis, output_project)
        if minimization:
            command += " -force_field OPLS3e "
        if match:
            command += " -match {} ".format(match)
        print(command)
        os.system(command)
            

def run_phase_screen(query_mol, database, hypotesis=None, minimization=True, match=None):
    print("\n")
    print("Pharmacophore Screening")
    print("------------------------\n")
    print("Input query file: {}".format(query_mol))
    print("Database to screen: {}".format(database))
    print("Initial hypotesis: {}".format(hypotesis))
    print("\n")
    query_mol_mae = ft.sdf_to_mae(query_mol)
    phase = PhaseScreen(query_mol_mae, database)
    if hypotesis:
        phase.hypotesis = hypotesis
    else:
        phase.create_pharmacophore_hypotesis()
    phase.run(minimization=minimization, match=match)
    

def parse_args(parser):
    parser.add_argument('--query_molec', type=str,
                        help='Reference molecule to create pharmacophores from')
    parser.add_argument('--database', type=str,
                        help='Database for pharmacophore screening')
    parser.add_argument('--hypotesis', type=str,
                        help='Hypotesis file (.phypo)', default=None)
    parser.add_argument('--nominimization', action="store_false",
                        help='Add Minimization')
    parser.add_argument('--match', type=int,
                        help='Minimum pharmacophores to match', default=None)

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description='Pharmacophore Screening')
    parse_args(parser)
    args = parser.parse_args()
    run_phase_screen(args.query_molec, args.database, hypotesis=args.hypotesis, minimization=args.nominimization, match=args.match)
    



from sklearn.decomposition import PCA
import umap


class ReduceDimension(object):

    def __init__(self, data, n_components):
        self.data = data
        self.n_components = n_components

    def run(self, method):
        if method == "umap":
            return self.compute_umap()
        elif method == "pca":
            return self.compute_pca()

    def compute_pca(self):
        pca = PCA(n_components=self.n_components)
        return pca.fit_transform(self.data)

    def compute_umap(self, neighbors=5, min_dist=0.2):
        reducer = umap.UMAP(n_neighbors=neighbors, min_dist=min_dist)
        return reducer.fit_transform(self.data)
        




import numpy as np
import umap
import matplotlib.pyplot as plt
import mpld3
from mpld3 import plugins

def histogram(point, interval=[0,1], n_bins=10):
    point = float(point)
    bins = np.linspace(interval[0], interval[1], num=n_bins)
    counts = [0] * (n_bins-1)
    for i in range(1, n_bins):
        if (point > bins[i-1]) and (point < bins[i]):
            counts[i-1] =+ 1
            break
        else:
            pass
    return counts, bins


def boxplot(X, title="Similarity box plot", output="box_plot.png"):
    fig1, ax1 = plt.subplots()
    ax1.set_title(title)
    ax1.boxplot(data)
    fig1.savefig(output)


def scatter_plot(X, Y, title="Scatter plot", output="scatter_plot.png"):
    fig1, ax1 = plt.subplots()
    ax1.set_title(title)
    ax1.scatter(X, Y)
    fig1.savefig(output)

def UMAP_plot(X, Y, neighbors=5, min_dist=0.2, title="Chamical Space plot", output="chemical_space_plot.png"):
    reducer = umap.UMAP(n_neighbors=neighbors, min_dist=min_dist)
    embedding = reducer.fit_transform(X)
    fig, ax = plt.subplots()
    ax.scatter(embedding[:, 0], embedding[:, 1])
    fig.gca().set_aspect('equal', 'datalim')
    ax.set_title(title)
    ax.legend()
    fig.savefig(output)
    return embedding

def interactive_map(x, y, svgs, color=None):
    fig, ax = plt.subplots()
    # Compute areas and colors
    if color is None:
        points = ax.scatter(x, y, c=color, alpha=0.75)
    else:
        points = ax.scatter(x, y, c=color, alpha=0.75, cmap="viridis")
    # This is key point for making tooltip!
    plt.colorbar(points)
    tooltip = plugins.PointHTMLTooltip(points, svgs)
    plugins.connect(fig, tooltip)
    mpld3.show(fig)



from rdkit import Chem
from rdkit.Chem import AllChem
import numpy as np


def postfilter_mols(molecule_query, mol_most_similars, atoms_to_grow, atoms_to_avoid, avoid_repetition):

    try:
        mols_after_filter = list(mol_most_similars); m_ref=[ m for m in molecule_query][0]
    except TypeError:
        mols_after_filter = list(mol_most_similars); m_ref=molecule_query
    if isinstance(m_ref, str): m_ref = next(Chem.SDMolSupplier(molecule_query[0]))

    if avoid_repetition:
        #Broken
        mol_most_similars = remove_duplicates(mol_most_similars)
    for m in reversed(mols_after_filter):
        if any(atoms_to_grow):
            if not is_bonded(m_ref, m.molecule, atoms_to_grow):
                mols_after_filter.remove(m)
                continue
        if any(atoms_to_avoid):
            if not is_not_bonded(m_ref, m.molecule, atoms_to_avoid):
                mols_after_filter.remove(m)
                continue
    return mols_after_filter

def remove_duplicates(molecules_duplicated):
    names = []
    mols_no_duplicates = []
    for mol in molecules_duplicated:
        name = mol.molecule.GetProp("_Name")
        if name in names:
            pass
        else:
            names.append(name)
            mols_no_duplicates.append(mol)
    return mols_no_duplicates

def retrieve_atom_info(m_ref, atoms):
    coords=[]; neighbors=[]; indexes=[]
    for atom in  m_ref.GetAtoms():
        if atom.GetIdx()+1 in atoms:
            indexes.append(atom.GetIdx())
            neighbors.append(len(atom.GetNeighbors()))
            coords.append(m_ref.GetConformer(0).GetPositions()[atom.GetIdx()])
    return coords, neighbors, indexes


def is_bonded(m_ref, mol, atoms):
    try:
        rmsd = Chem.rdMolAlign.GetBestRMS(mol, m_ref)
    except RuntimeError:
        try:
            rmsd = Chem.rdMolAlign.GetBestRMS(m_ref, mol)
        except RuntimeError:
            return False
    coords, neighbors, indexes = retrieve_atom_info(m_ref, atoms)
    for coord_ref, neighbour_ref, index_ref in zip(coords, neighbors, indexes):
        coords = mol.GetConformer(0).GetPositions()
        distances_to_atoms_from_ref = [np.linalg.norm(coord_ref - coord) for coord in coords]
        idx = np.argmin(distances_to_atoms_from_ref)
        min_distance_to_atomref = distances_to_atoms_from_ref[idx]
        if distances_to_atoms_from_ref[idx] == 0:
            return False
        atom = [ atom for i, atom in enumerate(mol.GetAtoms()) if i == idx][0]
        if len(atom.GetNeighbors()) > neighbour_ref:
            w=Chem.SDWriter("trial{}.sdf".format(min_distance_to_atomref))
            return True
    return False


def is_not_bonded(m_ref, mol, atoms):
    try:
        rmsd = Chem.rdMolAlign.GetBestRMS(mol, m_ref)
    except RuntimeError:
        try:
            rmsd = Chem.rdMolAlign.GetBestRMS(m_ref, mol)
        except RuntimeError:
            return False
    coords, neighbors, indexes = retrieve_atom_info(m_ref, atoms)
    for coord_ref, neighbour_ref, index_ref in zip(coords, neighbors, indexes):
        coords = mol.GetConformer(0).GetPositions()
        distances_to_atoms_from_ref = [np.linalg.norm(coord_ref - coord) for coord in coords]
        idx = np.argmin(distances_to_atoms_from_ref)
        min_distance_to_atomref = distances_to_atoms_from_ref[idx]
        if distances_to_atoms_from_ref[idx] == 0:
            return False
        atom = [ atom for i, atom in enumerate(mol.GetAtoms()) if i == idx][0]
        if len(atom.GetNeighbors()) > neighbour_ref:
            return False
    return True











from tqdm import tqdm
import numpy as np
from rdkit import Chem
from rdkit import DataStructs
from analogs_finder.search_methods import fingerprints as fps
from analogs_finder.search_methods import molecule as ml


def compute_similarity(mref, molecules, fp_type="DL"):
    fp_ref = fps.fingerprint(mref, fp_type)
    for i, m in enumerate(molecules):
        if m:
            yield ml.molec_properties(m, fp_ref=fp_ref, fp_type=fp_type)
        else:
            print("Molecule {}".format(i))

def compute_similarity_severl_fp(mref, molecules, fp_types=["DL", "circular", "torsions", "MACCS"], tresholds=[0.7, 0.4, 0.7, 0.4]):
    for i, m in tqdm(enumerate(molecules)):
        chosen = False
        for j, fp_type in enumerate(fp_types):
            fp_ref = fps.fingerprint(mref, fp_type=fp_type)
            if not chosen:
                if m:
                    molec = ml.molec_properties(m, fp_ref=fp_ref, fp_type=fp_type)
                    if molec.similarity > float(tresholds[j]):
                        chosen = True
                        yield molec
                else:
                    print("Molecule {}".format(i))
            else:
                break

def compute_similarity_several_mols(mrefs, molecules, fp_type="DL"):
    fp_refs = [ fps.fingerprint(mref, fp_type=fp_type) for mref in mrefs if mref ]
    for m in molecules:
        if m:
            molec = ml.molec_properties(m, fp_type=fp_type)
            molec.similarities = [DataStructs.FingerprintSimilarity(fp_ref, molec.fp) for fp_ref in fp_refs ]
            yield molec
        else:
            pass



from analogs_finder.search_methods import fingerprints as fps
from rdkit import Chem
from rdkit import DataStructs
from rdkit.Chem import Draw
from rdkit.Chem.Draw import rdMolDraw2D
from rdkit.Chem import AllChem


class Molecule(object):

    def __init__(self, molecule):
        self.molecule = molecule
        self.fp = None
        self.similarity = None


    def compute_similarity(self, fp_ref):
        self.fp_ref = fp_ref
        self.similarity = DataStructs.FingerprintSimilarity(self.fp_ref, self.fp)
        return self.similarity


    def compute_fp(self, fp_type):
        self.fp_type = fp_type
        self.fp = fps.fingerprint(self.molecule, fp_type=self.fp_type)
        return self.fp

    def to_svg(self, molSize=(300,300), kekulize=True):
        mc = Chem.Mol(self.molecule.ToBinary())
        if kekulize:
            try:
                Chem.Kekulize(mc)
            except:
                mc = Chem.Mol(self.molecule.ToBinary())
        if not mc.GetNumConformers():
            mc = AllChem.Compute2DCoords(mc)
        AllChem.Compute2DCoords(mc)
        drawer = rdMolDraw2D.MolDraw2DSVG(molSize[0],molSize[1])
        drawer.DrawMolecule(mc)
        drawer.FinishDrawing()
        svg = drawer.GetDrawingText()
        return svg.replace('svg:','')
        

def molec_properties(m, fp_ref=None, fp_type=None):
    molec = Molecule(m)
    if fp_type:
        molec.compute_fp(fp_type)
    if fp_ref:
        molec.compute_similarity(fp_ref)
    return molec



from analogs_finder.analysis import analysis_dataset as an
import types
import argparse
import sys
from functools import partial
import collections
from tqdm import tqdm
import numpy as np
from rdkit import Chem
from rdkit import DataStructs
from rdkit.Chem.Fingerprints import FingerprintMols
from multiprocessing import Pool
from analogs_finder.search_methods import loader as ld
from analogs_finder.helpers import postfilter as pt
from analogs_finder.helpers import helpers as hp



def query_database(database, molecules, n_structs=500, combi_subsearch=False, most_similars=False, substructure=False, output="similars.sdf", hybrid=None, treshold=0.7, avoid_repetition=False, fp_type="DL", turbo=False, neighbours=5, analysis_dataset=False, test=False, dim_type="pca", atoms_to_grow=[], atoms_to_avoid=[], only_postfilter=False):

    #Initial checks
    assert type(database) == str, "database must be a unique sdf file"
    assert type(molecules) == list, "query molecule must be a list of a single or multiple sdf files"

    # Set tanimoto threshold and fp type
    if type(treshold) == list and type(fp_type) == list:
        if len(treshold) == 1 and len(fp_type) == 1:
           treshold = treshold[0]
           fp_type = fp_type[0] 

    # Load database
    molecules_db= Chem.SDMolSupplier(database)

    # Perform database analysis
    if analysis_dataset:
        print("Analysing dataset...")
        molecule_query = next(Chem.SDMolSupplier(molecules[0]))
        an.main(molecule_query, molecules_db, test=test, dim_type=dim_type)
        return

    # Load query molecule
    molecule_query = ld.load_query_molecule(molecules, most_similars, turbo,
            combi_subsearch, substructure, hybrid, treshold, fp_type, only_postfilter) 

    # Load method to use
    mol_most_similars = ld.load_method_to_use(most_similars, turbo, substructure, combi_subsearch,
        hybrid, fp_type, treshold, n_structs, molecule_query, molecules_db, neighbours,
        only_postfilter)

    #If output
    if mol_most_similars:
        mol_most_similars = list(mol_most_similars)

        #Write molecules before filtering
        n_mol_found_before_filter = hp.molecules_to_sdf(mol_most_similars, "before_filter_" + output)
        print("Number of found molecules {} before filter".format(n_mol_found_before_filter))

        #Filter molecules
        mols_after_filter = pt.postfilter_mols(molecule_query, mol_most_similars, atoms_to_grow, atoms_to_avoid, avoid_repetition)

        #Write molecules after filtering
        n_mol_found_after_filter = hp.molecules_to_sdf(mols_after_filter, output)
        print("Number of found molecules {} after filter".format(n_mol_found_after_filter))
        #p = Pool(processes=20)
        #data = p.map(partial(search_most_similars, molecule_query=molecule_query, n_structs=n_structs), molecules_db) 
    return n_mol_found_after_filter


def add_args(parser):
    parser.add_argument('database', type=str, help="database to query")
    parser.add_argument('molecules', nargs="+", help="molecule to query")
    parser.add_argument('--n', type=int, help="Number of structures tp retrieve", default=500)
    parser.add_argument('--tresh', nargs="+", help="Tanymoto similarity tresholt default=0.7", default=[0.7])
    parser.add_argument('--output', type=str, help="Name of output file", default="analogs.sdf")
    parser.add_argument('--sb', action="store_true", help="Get the n most similar structs")
    parser.add_argument('--substructure', action="store_true", help="Get all the structures with a certain substructure")
    parser.add_argument('--combi_subsearch', action="store_true", help="Get almost on of the substructures in each sdf file")
    parser.add_argument('--avoid_repetition', action="store_true", help="Allow to have the same molecule name several times in the final sdf file")
    parser.add_argument('--hybrid', type=str, help="Hybird model between similarity and substructure search")
    parser.add_argument('--fp_type', nargs="+", help="Fingerprint type to use [DL, circular, torsions, MACCS]", default=["DL"])
    parser.add_argument('--turbo', action="store_true", help="Run similarity for your reference structure and N most similar neighbours")
    parser.add_argument('--neighbours', type=int, help="Number of neighbours in turbo search. Ignored if not flag --turbo")
    parser.add_argument('--analysis_dataset', action="store_true", help="Retrieve the similarity distribution of your dataset")
    parser.add_argument('--test', action="store_true", help="Whether to run test or not")
    parser.add_argument('--dim_type', type=str, help="Dimensionallity reduction mothod to use when analyzing", default="pca")
    parser.add_argument('--atoms_to_grow', nargs="+", type=int, help="Atoms you want to grow towards", default=[])
    parser.add_argument('--atoms_to_avoid', nargs="+", type=int, help="Atoms you want to avoid growing to", default=[])
    parser.add_argument('--only_postfilter', action="store_true", help="Postfilter results from a previous analog search")

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description='Find analogs to a query molecule on your private database')
    add_args(parser)
    args = parser.parse_args()
    query_database(args.database, args.molecules, n_structs=args.n, most_similars=args.sb, 
          combi_subsearch=args.combi_subsearch, substructure=args.substructure, output=args.output, treshold=args.tresh, avoid_repetition=args.avoid_repetition, hybrid=args.hybrid, fp_type=args.fp_type, turbo=args.turbo, neighbours=args.neighbours, analysis_dataset=args.analysis_dataset, test=args.test, dim_type=args.dim_type, atoms_to_grow=args.atoms_to_grow, atoms_to_avoid=args.atoms_to_avoid, only_postfilter=args.only_postfilter)



from rdkit import Chem
from analogs_finder.search_methods import methods as mt
from analogs_finder.search_methods import fusion as fs
from analogs_finder.search_methods import molecule as ml



def load_query_molecule(molecules, most_similars, turbo, combi_subsearch, substructure, hybrid, treshold, fp_type, only_postfilter):
    if only_postfilter:
         molecule_query = next(Chem.SDMolSupplier(molecules[0]))
    elif most_similars:
        molecule_query = next(Chem.SDMolSupplier(molecules[0]))
    elif turbo:
        molecule_query = next(Chem.SDMolSupplier(molecules[0]))
    elif combi_subsearch:
        molecule_query = molecules
    elif substructure:
        molecule_query = Chem.SDMolSupplier(molecules[0])
    elif hybrid:
        molecule_query = next(Chem.SDMolSupplier(molecules[0]))
    elif type(treshold) == list and type(fp_type) == list:
        molecule_query = next(Chem.SDMolSupplier(molecules[0]))
    elif treshold:
        molecule_query = next(Chem.SDMolSupplier(molecules[0]))
    return molecule_query


def load_method_to_use(most_similars, turbo, substructure, combi_subsearch, 
        hybrid, fp_type, treshold, n_structs, molecule_query, molecules_db,
        neighbours, only_postfilter): 
    mol_most_similars = None
    if only_postfilter:
        mol_most_similars = [ml.Molecule(m) for m in molecules_db if m]
    elif most_similars:
        mol_most_similars  = mt.search_most_similars(molecule_query, molecules_db, n_structs, fp_type=fp_type)
    elif turbo:
        mol_most_similars = fs.turbo_similarity(molecule_query, molecules_db, neighbours=neighbours, treshold=treshold, fp_type=fp_type)
    elif substructure:
        mol_most_similars  = mt.search_substructure(molecule_query, molecules_db)
    elif combi_subsearch:
        mol_most_similars = mt.combi_substructure_search(molecule_query, molecules_db) 
    elif hybrid and treshold:
        mol_most_similars = mt.most_similar_with_substructure(molecule_query, molecules_db, hybrid, treshold, fp_type=fp_type)
    elif type(treshold) == list and type(fp_type) == list:
        mol_most_similars = mt.search_similarity_tresh_several_fp(molecule_query, molecules_db, tresholds=treshold, fp_types=fp_type)
    elif treshold:
        mol_most_similars  = mt.search_similarity_tresh(molecule_query, molecules_db, treshold, fp_type=fp_type)
    return mol_most_similars
        




# Logging
import logging

try:  # Python 2.7+
    from logging import NullHandler
except ImportError:
    class NullHandler(logging.Handler):

        def emit(self, record):
            pass

logging.getLogger(__name__).addHandler(NullHandler())

__author__ = 'Daniel Soler'
__copyright__ = '2019, NostrumBioDiscovery'
__url__ = 'https://github.com/danielSoler93'
__title__ = 'AnalogFinder'
__description__ = ('AnalogFinder: Given a query molecule and some database find analogs')




import setuptools


setuptools.setup(name='analogs_finder',
      version="1.1.0",
      url = "https://github.com/nostrumbiodiscovery/analogs_finder", 
      description='Retrieve analogs given a query molecule and a database.',
      author='Daniel Soler',
      author_email='daniel.soler@nostrumbiodiscovery.com',
      packages=setuptools.find_packages(),
      install_requires=["mpld3", "tqdm"],
      classifiers=[
       "Programming Language :: Python :: 3",
       "License :: OSI Approved :: MIT License",
       "Operating System :: OS Independent",
       ],
     )



import numpy as np
import matplotlib.pyplot as plt
from tqdm import tqdm
from rdkit import DataStructs
from analogs_finder.analysis import plot as pt
from analogs_finder.search_methods import similarity as sim
from analogs_finder.search_methods import molecule as ml
from analogs_finder.analysis import dimensionallity as dm


def main(molecule_query, molecules_db, n_bins=100, fp_types=["DL", "MACCS", "circular"], test=None, dim_type="umap"):

    #Analysis code
    for fp_type in fp_types:

        # Retrieve fingerprints and similarity for analysis
        counts = []
        svg = [None] * 100
        fp = [None] * 100
        similarity = [0] * 100
        for m in tqdm(sim.compute_similarity(molecule_query, molecules_db, fp_type)):
            #Build similarity counts
            counts_step, bins = pt.histogram(m.similarity, n_bins=n_bins)
            counts = np.add(counts, counts_step) if len(counts) != 0 else counts_step

            #Build fingeprints
            idx = np.argmin(similarity)
            less_similar = similarity[idx]
            if m.similarity > less_similar:
                fp[idx] = m.fp
                similarity[idx] = m.similarity
                svg[idx] = m.to_svg()

        #Plot similarity distribution
        if not test:
            fig, ax = plt.subplots()
            ax.hist(bins[1:], bins, weights=counts)
            plt.savefig("similarity_hist_{}.png".format(fp_type))

        #Retrieve similarity distribution data
        values = np.vstack([bins[1:], counts]).T
        np.savetxt("counts_{}.txt".format(fp_type), values,  fmt=['%f','%d'])

        if fp_type == "circular":
            #Add reference molecule
            mref = ml.molec_properties(molecule_query, fp_type=fp_type)
            fp.insert(0, mref.fp)
            similarity.insert(0, 1)
            svg.insert(0, mref.to_svg())

            #Clean results
            svg = [ value for value in svg if value ]
            fp = [ value for value in fp if value ]
            similarity = [ value for value in similarity if value!= 0 ]

            #Reduce Dimension
            X = np.asarray([ fp2arr(f) for f in fp ])
            embedding = dm.ReduceDimension(X, 2).run(dim_type)

            #Plot Reduce Dimension graph
            if not test:
                pt.interactive_map(embedding[:, 0], embedding[:, 1], svg, color=np.array(similarity))

def fp2arr(fp):
    arr = np.zeros((1,))
    DataStructs.ConvertToNumpyArray(fp,arr)
    return arr




from rdkit import Chem



def molecules_to_sdf(molecules, output):
    w = Chem.SDWriter(output)
    n_mol_found = 0
    for molecule in molecules: 
        try:
            w.write(molecule.molecule)
        except AttributeError:
            w.write(molecule)
        n_mol_found += 1
    return n_mol_found
